<!DOCTYPE html>
<html ng-app="ODKewa" ng-cloak> <!-- ng-cloak -->
<head>
   <title>ODKewa</title>
<!--    <script type="text/javascript" src="{{ url_for('static', filename='jquery-3.3.1.min.js') }}"></script> -->
   <link rel="shortcut icon" href="{{ url_for('static', filename='favicon.ico') }}"/>
   <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/css/bootstrap.min.css">
   <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/select2-bootstrap-css/1.4.6/select2-bootstrap.min.css">
   <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.3/css/select2.min.css">
   <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/font-awesome/4.5.0/css/font-awesome.min.css">
   <link rel="stylesheet" href="https://cdn.datatables.net/1.10.10/css/dataTables.bootstrap.min.css">
   <link rel="stylesheet" href="https://cdn.datatables.net/fixedcolumns/3.2.0/css/fixedColumns.bootstrap.min.css">
   <script src="https://ajax.googleapis.com/ajax/libs/jquery/1.11.3/jquery.min.js"></script>
   <script src="https://code.jquery.com/jquery-1.12.0.min.js"></script>
   <script src="https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.3/js/select2.full.js"></script>
   <script src="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/js/bootstrap.min.js"></script>
   <script src="https://cdn.datatables.net/1.10.10/js/jquery.dataTables.min.js"></script>
   <script src="https://cdn.datatables.net/1.10.10/js/dataTables.bootstrap.min.js"></script>
   <script src="https://cdn.datatables.net/fixedcolumns/3.2.0/js/dataTables.fixedColumns.min.js"></script>
   <script type="text/javascript" src="{{ url_for('static', filename='angular.min.js') }}"></script>
   <script type="text/javascript" src="{{ url_for('static', filename='ng-file-upload-shim.min.js') }}"></script>
   <script type="text/javascript" src="{{ url_for('static', filename='ng-file-upload.min.js') }}"></script>

   <style type="text/css">
      [ng\:cloak], [ng-cloak], .ng-cloak {
         display: none;
      }
      .rowgroup {
         border-style: solid none none none;
         border-width: thin; 
         border-color: #dddddd;
         background-color: none;
         padding-top: 1em;
         padding-bottom: 1em;
      }
      span.err { color: #f00 }
      .highlight { 
          background:#F84072;
          border: 2px red solid;
      }
      .form-group.required .control-label:after {
         content:"*";
         color:red;
      }
      .thumb {
          width: 24px;
          height: 24px;
          float: none;
          position: relative;
          top: 7px;
      }

/*    .group {
         background: green;
         opacity: 0.9;
         color: red;
      }
      .repeat {
         background: yellow;
         opacity: 0.9;
         color: blue;
      }*/
   </style>
</head>
<body>
   {{ form|safe }}
</body>
</html>
